@@ -22,6 +22,6 @@ class RectangularSlit(DOE):
 
         t = bd.where((((xx > (self.x0 - self.width / 2)) & (xx < (self.x0 + self.width / 2)))
                         & ((yy > (self.y0 - self.height / 2)) & (yy < (self.y0 + self.height / 2)))),
-                        bd.ones_like(self.xx), bd.zeros_like(self.xx))
+                        bd.ones_like(xx), bd.zeros_like(xx))
 
         return t
